package de.afb.maven;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;

/**
 * 
 * This plugin came to live due to the following bug in glassfish: 
 * 
 * https://glassfish.dev.java.net/issues/show_bug.cgi?id=11402
 * 
 * It's sole purpose is to remove dots from within filenames. 
 * 
 * @author Harald Entner
 * 
 * @goal postProcess
 * @requiresDependencyResolution compile
 */
public class EarPostProcessor extends AbstractMojo {

    /**
     * @parameter expression="${postProcess.targetDirectory}"
     */
    private String targetDirectory;
    /**
     * @parameter expression="${postProcess.nameOfEar}"
     */
    private String nameOfEar;
    private static final String tmpDirectory = System.getProperty("java.io.tmpdir")
            + "/mavenPostProcessor";

    public static void main(String[] args) {
        EarPostProcessor processor = new EarPostProcessor();
        try {
            processor.nameOfEar = "ADG-1.0-SNAPSHOT.ear";
            processor.targetDirectory = "/home/hen/eclipse/workspace/mavenbranch/Server/ADG-EAR/target";
            processor.execute();
        } catch (MojoExecutionException ex) {
            Logger.getLogger(EarPostProcessor.class.getName()).log(
                    Level.SEVERE, null, ex);
        } catch (MojoFailureException ex) {
            Logger.getLogger(EarPostProcessor.class.getName()).log(
                    Level.SEVERE, null, ex);
        }
    }

    public void execute() throws MojoExecutionException, MojoFailureException {
        System.out.println("Running Ear Post Processor on " + targetDirectory + "/" + nameOfEar);

        File originalEar = new File(targetDirectory + "/" + nameOfEar);
        File newEar = new File(originalEar + ".copy");

        try {
            ZipFile zipFile = new ZipFile(originalEar);
            ZipInputStream in = new ZipInputStream(new FileInputStream(originalEar));
            ZipOutputStream out = new ZipOutputStream(new FileOutputStream(newEar));
            ZipEntry entry;

            while ((entry = in.getNextEntry()) != null) {
                // remove all dots in filenames
                ZipEntry newEntry = new ZipEntry(removeDots(entry.getName()));
                System.out.println("Renaming entry:" + entry.getName() + " to " + newEntry.getName());
                out.putNextEntry(entry);

                byte[] buffer = new byte[1024];
                int bytesRead;
                BufferedInputStream bis = new BufferedInputStream(zipFile.getInputStream(entry));
                while ((bytesRead = bis.read(buffer)) != -1) {
                    out.write(buffer, 0, bytesRead);
                }
                bis.close();
                out.closeEntry();
            }

            in.close();
            out.close();
            
            
            // delete original and replace it with new file 
            originalEar.delete();
            newEar.renameTo(originalEar);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Replaces all dots contained in the String except the
     * last one.
     *
     * @param name
     * @return a String that does only contain one dot.
     */
    private String removeDots(String name) {
        int lastIndex = name.lastIndexOf(".");

        String newName = name;
        while (newName.indexOf(".") != lastIndex) {
            newName = newName.replaceFirst("\\.", "-");
        }
        return newName;
    }

    private void copyFiles(File in, File out) throws IOException {
        if (!out.exists()) {
            out.createNewFile();
        }

        FileReader reader = new FileReader(in);
        FileWriter writer = new FileWriter(out);
        int c;
        while ((c = reader.read()) != -1) {
            writer.write(c);
        }
        reader.close();
        writer.close();

    }
}
